package edu.uiuc.edsg;

/**
 * Abstract base class for link nodes with two children
 */
public abstract class BinaryLink implements IIterationLink {
	
	private IIterationLink _left;
	private IIterationLink _right;
	
	public IIterationLink getLeft() {
		return _left;
	}
	
	public IIterationLink getRight() {
		return _right;
	}
	
	public BinaryLink(IIterationLink left, IIterationLink right) {
		_left = left;
		_right = right;
	}
	
	/**
	 * Resets this link and all child links to their initial state. Overridable.
	 *
	 */
	public void reset() {
		_left.reset();
		_right.reset();
	}
	
	/**
	 * @return true if this link can be iterated
	 */
	public abstract boolean hasNext();
	
	/**
	 * Iterates this link. The specific form of iteration is determined by the derived class
	 *
	 */
	public abstract void next();
	
	/**
	 * Used to create a new instance of this type of BinaryLink without knowing the concrete type
	 */
	public abstract BinaryLink createNewInstance(IIterationLink left, IIterationLink right);
	
}
