package edu.uiuc.edsg.samples;

import edu.uiuc.edsg.*;

public class ListGenerator<ValueType> extends CompositeGenerator<List<ValueType>> implements Standinable<List<ValueType>> {
	private int length;
	private ValueGenCallback<ValueType> valueGenCallback;
	private IGenerator<ValueType> valueGenerator;
	private Chain<List<ValueType>> nodeGenerator;
	
	public ListGenerator(ValueGenCallback<ValueType> valueGenCallback, int length) {
		if (length < 1) {
			throw new java.lang.IllegalArgumentException("List length must be >= 1");
		}
		
		this.valueGenCallback = valueGenCallback;
		this.valueGenerator = valueGenCallback.newValueGenerator();
		this.length = length;
		
		nodeGenerator = new Chain<List<ValueType>>();
		nodeGenerator.add(new Literal<List<ValueType>>(null));
		if (length > 1) {
			nodeGenerator.add(new Standin<List<ValueType>>(this));
		}
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		return LinkCreator.createCombinatoricLink(nodeGenerator, valueGenerator);
	}

	@Override
	protected List<ValueType> generateCurrent() {
		return new List<ValueType>(valueGenerator.current(), nodeGenerator.current());
	}
	
	public ListGenerator<ValueType> expandStandin() {
		return new ListGenerator<ValueType>(valueGenCallback, length - 1);
	}
	
	public static interface ValueGenCallback<ValueType> {
		public IGenerator<ValueType> newValueGenerator();		
	}
	
}
