package edu.uiuc.edsg.samples;

import edu.uiuc.edsg.*;

public class TreeGenerator<ValueType> extends CompositeGenerator<Tree<ValueType>> implements Standinable<Tree<ValueType>> {
	private int depth;
	private IGenerator<ValueType> valueGenerator;
	private Chain<Tree<ValueType>> leftNodeGenerator;
	private Chain<Tree<ValueType>> rightNodeGenerator;
	private ValueGenCallback<ValueType> valueGenCallback;	

	public TreeGenerator(ValueGenCallback<ValueType> valueGenCallback, int depth) {
		if (depth < 1) {
			throw new java.lang.IllegalArgumentException("Tree depth must be >= 1");
		}
		
		this.valueGenCallback = valueGenCallback;
		this.valueGenerator = valueGenCallback.newValueGenerator();
		this.depth = depth;		
		this.leftNodeGenerator = makeNodeGenerator(depth);
		this.rightNodeGenerator = makeNodeGenerator(depth);
	}

	private Chain<Tree<ValueType>> makeNodeGenerator(int depth) {
		Chain<Tree<ValueType>> nodeGen = new Chain<Tree<ValueType>>();
		nodeGen.add(new Literal<Tree<ValueType>>(null));
		if (depth > 1) {
			nodeGen.add(new Standin<Tree<ValueType>>(this));
		}
		return nodeGen;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		return LinkCreator.createCombinatoricLink(
				valueGenerator, 
				leftNodeGenerator, 
				rightNodeGenerator);
	}
	
	@Override
	protected Tree<ValueType> generateCurrent() {
		return new Tree<ValueType>(
				valueGenerator.current(), 
				leftNodeGenerator.current(), 
				rightNodeGenerator.current());
	}	

	/**
	 * Returns a recursed tree generator 
	 */
	public TreeGenerator<ValueType> expandStandin() {
		return new TreeGenerator<ValueType>(valueGenCallback, depth - 1);
	}	
	
	public static interface ValueGenCallback<ValueType> {
		public IGenerator<ValueType> newValueGenerator();
	}
}
