package edu.uiuc.edsg.test;

import org.junit.*;
import edu.uiuc.edsg.*;
import edu.uiuc.edsg.samples.Pair;
import edu.uiuc.edsg.samples.PairGenerator;

public class FilteredGeneratorTest {
	final static Integer TEST_VALUE = 2;
	Chain<Integer> testSequence = new Chain<Integer>(1, TEST_VALUE, 3);
	private FilteredGenerator<Integer> filteredSequence;
	
	@Before
	public void setUp() {
		filteredSequence = new FilteredGenerator<Integer>(testSequence);
	}
	
	@Test
	public void noPredicates() {
		// filtered iteration should echo test sequence
		GeneratorTestUtil.testBasicFunctionality(filteredSequence, "1", TEST_VALUE.toString(), "3");
	}
	
	@Test
	public void filterByPredicate() {				
		// filter by test value
		filteredSequence.registerPredicate(
				new Predicate<Integer> () {
					public boolean evaluate(Integer testObject) {
						return testObject.equals(TEST_VALUE);
					}
				});
		
		GeneratorTestUtil.testBasicFunctionality(filteredSequence, TEST_VALUE.toString());		
	}
	
	@Test
	public void multiplePredicates() {				
		// Accept only those values lower than testValue + 1
		filteredSequence.registerPredicate(
				new Predicate<Integer> () {
					public boolean evaluate(Integer testObject) {
						return testObject.intValue() < TEST_VALUE + 1;
					}
				});
		// Accept only those values greater than testValue - 1
		filteredSequence.registerPredicate(
				new Predicate<Integer> () {
					public boolean evaluate(Integer testObject) {
						return testObject.intValue() > TEST_VALUE - 1;
					}
				});
		
		// Make sure only the test value was generated
		GeneratorTestUtil.testBasicFunctionality(filteredSequence, TEST_VALUE.toString());		
	}
	
	@Test
	public void noPredicatesSatisfied() {
		filteredSequence.registerPredicate(
				new Predicate<Integer>() {
					public boolean evaluate(Integer testObject) {						
						return false; // no values are accepted
					}
				});
		GeneratorTestUtil.testBasicFunctionality(filteredSequence, 0);
	}
	
	@Test
	public void linkedCombinatorically() {		
		// filter by test value
		filteredSequence.registerPredicate(
				new Predicate<Integer> () {
					public boolean evaluate(Integer testObject) {
						return testObject.equals(TEST_VALUE);
					}
				});
		Chain<Integer> dummySequence = new Chain<Integer>(1, 2, 3);
		
		// make filtered generator part of a larger generator 
		PairGenerator<Integer, Integer> pair = new PairGenerator<Integer, Integer>(dummySequence, filteredSequence);
		GeneratorTestUtil.testBasicFunctionality(pair, 
				"[1," + TEST_VALUE.toString() + ']',
				"[2," + TEST_VALUE.toString() + ']',
				"[3," + TEST_VALUE.toString() + ']');
		
		// and in reverse 
		PairGenerator<Integer, Integer> pair2 = new PairGenerator<Integer, Integer>(filteredSequence, dummySequence);
		GeneratorTestUtil.testBasicFunctionality(pair2, 
				'[' + TEST_VALUE.toString() + ",1]",
				'[' + TEST_VALUE.toString() + ",2]",
				'[' + TEST_VALUE.toString() + ",3]");
	}
}
