package edu.uiuc.edsg.test;

import org.junit.*;

import edu.uiuc.edsg.PrototypeGenerator;

public class PrototypeGeneratorTest {

	public static class TestClass {
		public int i;
		public boolean b;
		private char c; // uses accessor methods
		private long l; // uses accessor methods
		private Object onlySetter = null;
		private Object onlyGetter = null;
		
		public void setC(char c) {
			this.c = c;
		}
		public char getC() {
			return this.c;
		}

		public long getL() {
			return l;
		}
		public void setL(long l) {
			this.l = l;
		}
		
		public void setOnlySetter(Object value) {
			Assert.fail();
		}
		
		public Object getOnlyGetter() {
			Assert.fail();
			return null;
		}
		
		public TestClass() {
			i = 0;
			b = false;
			c = '\0';
			l = 0;
		}
		public TestClass(int i, boolean b, char c, long l) {
			this.i = i;
			this.b = b;
			this.c = c;
			this.l = l;
		}
		@Override
		public String toString() {
			return i + ", " + b + ", " + c + ", " + l;
		}
	}

	PrototypeGenerator<TestClass> protoGen;

	@Before
	public void setUp() {
		TestClass proto1 = new TestClass(1, true, 'a', -1);
		TestClass proto2 = new TestClass(2, false, 'b', -2);
		
		protoGen = new PrototypeGenerator<TestClass>(TestClass.class);
		protoGen.addPrototype(proto1);
		protoGen.addPrototype(proto2);	
	}

	@Test
	public void noCombinatoric() {		
		GeneratorTestUtil.testBasicFunctionality(protoGen, 
				"1, true, a, -1", 
				"2, false, b, -2");
	}
	
	@Test
	public void singleCombinatoric() throws SecurityException, NoSuchFieldException {
		protoGen.setCombinatoric("i");
		GeneratorTestUtil.testBasicFunctionality(protoGen, 
				"1, true, a, -1", 
				"1, false, b, -2", 
				"2, true, a, -1", 
				"2, false, b, -2");
	}

	@Test
	public void doubleCombinatoric() throws SecurityException, NoSuchFieldException {
		protoGen.setCombinatoric("i");
		protoGen.setCombinatoric("c");
		GeneratorTestUtil.testBasicFunctionality(protoGen,
				"1, true, a, -1",
				"1, false, a, -2",
				"1, true, b, -1",
				"1, false, b, -2",
				"2, true, a, -1",
				"2, false, a, -2",
				"2, true, b, -1",
				"2, false, b, -2");
	}
	
	@Test
	public void tripleCombinatoric() throws SecurityException, NoSuchFieldException {
		protoGen.setCombinatoric("i");
		protoGen.setCombinatoric("c");
		protoGen.setCombinatoric("l");
		GeneratorTestUtil.testBasicFunctionality(protoGen, 2 * 2 * 2 * 2);
	}
	
	@Test
	public void allCombinatoric() throws SecurityException, NoSuchFieldException {
		protoGen.setCombinatoric("i");
		protoGen.setCombinatoric("b");
		protoGen.setCombinatoric("c");
		protoGen.setCombinatoric("l");
		GeneratorTestUtil.testBasicFunctionality(protoGen, 2 * 2 * 2 * 2);
	}
}
