package edu.uiuc.edsg.test;

import org.junit.*;

import edu.uiuc.edsg.*;
import edu.uiuc.edsg.samples.*;

public class SimpleWrapperTest {	
	Chain<Integer> targetGenerator = new Chain<Integer>(1, 2);
	static final int TARGET_COUNT = 2;	
	SimpleWrapper<Integer> wrapper = new SimpleWrapper<Integer>(targetGenerator);
	// Wrapper count will always be 1 because it acts like a literal
	
	@Before()
	public void setup() {
		targetGenerator.reset();
		wrapper.reset();
	}
	
	@Test() 
	public void explicitIteration() {		
		int count = 0;
		while (targetGenerator.hasNext()) {
			Assert.assertEquals(targetGenerator.next(), wrapper.next());
			Assert.assertEquals(targetGenerator.current(), wrapper.current());			
			count++;
		}
		Assert.assertEquals(TARGET_COUNT, count);
	}
	
	@Test(expected=IterationException.class) 
	public void invalidOrder() {
		// wrapper must come before the target, so this order will throw an exception when iterated
		IIterationLink link = new CombinatoricLink(
				new GeneratorLink(targetGenerator), 
				new GeneratorLink(wrapper));
		link.next();
		link.reset();
	}	
	
	@Test()
	public void basicDependence() {
		IGenerator<Integer> target = new Chain<Integer>(1,2,3);
		IGenerator<Integer> wrapper = new SimpleWrapper<Integer>(target);		
		// the left value depends on the right
		PairGenerator<Integer, Integer> dependentPair = new PairGenerator<Integer, Integer>(wrapper, target);
		GeneratorTestUtil.testBasicFunctionality(dependentPair,
				"[1,1]", 
				"[2,2]", 
				"[3,3]");				
	}
	
	@Test @Ignore
	public void dependentGenerators() {
		
		// NOTE: known test failure. Brett 4/16/07
		
		Chain<Integer> target1 = new Chain<Integer>(1,2);
		Chain<Character> target2 = new Chain<Character>('a','b','c');
				
		PairGenerator<Integer, Character> pairGen = new PairGenerator<Integer, Character>(target1, target2);
		// wrappedPair should always reflect the value of pairGen
		PairGenerator<Integer, Character> wrappedPair = new PairGenerator<Integer, Character>(
				new SimpleWrapper<Integer>(target1),
				new SimpleWrapper<Character>(target2));
		
		// Note that the generator order matters. The wrappers must come before their targets.
		PairGenerator<Pair<Integer, Character>, Pair<Integer,Character>> mainGen = 
			new PairGenerator<Pair<Integer,Character>, Pair<Integer,Character>>(pairGen, wrappedPair);
			
		GeneratorTestUtil.testBasicFunctionality(mainGen, 
				"[[1,a],[1,a]]",
				"[[2,a],[2,a]]",
				"[[1,b],[1,b]]",
				"[[2,b],[2,b]]",
				"[[1,c],[1,c]]",
				"[[2,c],[2,c]]");		
	}
	
	@Test(expected=IterationException.class) @Ignore
	public void dependentGenerators_InvalidOrder() {
		
		// NOTE: known test failure. Brett 4/16/07
		
		Chain<Integer> target1 = new Chain<Integer>(1,2);
		Chain<Character> target2 = new Chain<Character>('a','b','c');
				
		PairGenerator<Integer, Character> pairGen = new PairGenerator<Integer, Character>(target1, target2);
		// wrappedPair should always reflect the value of pairGen
		PairGenerator<Integer, Character> wrappedPair = new PairGenerator<Integer, Character>(
				new SimpleWrapper<Integer>(target1),
				new SimpleWrapper<Character>(target2));
		
		// Note that the generator order matters. The wrappers must come before their targets.
		PairGenerator<Pair<Integer, Character>, Pair<Integer,Character>> mainGen = 
			new PairGenerator<Pair<Integer,Character>, Pair<Integer,Character>>(wrappedPair, pairGen); // wrong order		
			
		mainGen.reset();
		mainGen.next(); // throws exception
	}
}
