package edu.uiuc.astgen.basic;

import java.util.ArrayList;

import org.eclipse.jdt.core.dom.ConstructorInvocation;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.PrimitiveType;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.edsg.IGenerator;
import edu.uiuc.edsg.IIterationLink;
import edu.uiuc.edsg.LinkCreator;

public class ConstructorInvocationGenerator extends ASTNodeGeneratorBase<ConstructorInvocation>{
	private ArrayList<IGenerator<? extends Expression>> arguments;
	
	public ConstructorInvocationGenerator() {
		super();
		arguments = new ArrayList<IGenerator<? extends Expression>>();
	}
	
	public void addArgumentGenerator(IGenerator<? extends Expression> argumentGen) {
		arguments.add(argumentGen);
	}	

	@Override
	protected ConstructorInvocation generateCurrent() {
		ConstructorInvocation constructorInvocation = super.getAST().newConstructorInvocation();
		for (IGenerator<? extends Expression> argumentGen : arguments) {
			Expression argument = argumentGen.current();
			if (argument != null) {
				constructorInvocation.arguments().add(argument);
			}
		}		
		return constructorInvocation;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator linkCreator = new LinkCreator(LinkCreator.LinkType.COMBINATORIC);
		linkCreator.addAll(arguments);
		return linkCreator.getLink();
	}

	public static ConstructorInvocationGenerator getNewDefaultConstructorInvocationGenerator(){
		NumberLiteralGenerator numberGen = new NumberLiteralGenerator(1, PrimitiveType.BYTE);
		StringLiteralGenerator stringGen = StringLiteralGenerator.getNewDefaultStringLiteralGenerator();
		ConstructorInvocationGenerator constructorInvocationGen = new ConstructorInvocationGenerator();	
		constructorInvocationGen.addArgumentGenerator(numberGen);
		constructorInvocationGen.addArgumentGenerator(stringGen);
		return constructorInvocationGen;
	}
}
