package edu.uiuc.astgen.basic;

import java.util.ArrayList;

import org.eclipse.jdt.core.dom.ConstructorInvocation;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.SuperConstructorInvocation;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.edsg.IGenerator;
import edu.uiuc.edsg.IIterationLink;
import edu.uiuc.edsg.LinkCreator;

public class SuperConstructorInvocationGenerator extends ASTNodeGeneratorBase<SuperConstructorInvocation>{
	private ArrayList<IGenerator<? extends Expression>> arguments;
	
	public SuperConstructorInvocationGenerator() {
		super();
		arguments = new ArrayList<IGenerator<? extends Expression>>();
	}
	
	public void addArgumentGenerator(IGenerator<? extends Expression> argumentGen) {
		arguments.add(argumentGen);
	}	

	@Override
	protected SuperConstructorInvocation generateCurrent() {
		SuperConstructorInvocation superConstructorInvocation = super.getAST().newSuperConstructorInvocation();
		for (IGenerator<? extends Expression> argumentGen : arguments) {
			Expression argument = argumentGen.current();
			if (argument != null) {
				superConstructorInvocation.arguments().add(argument);
			}
		}		
		return superConstructorInvocation;
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator linkCreator = new LinkCreator(LinkCreator.LinkType.COMBINATORIC);
		linkCreator.addAll(arguments);
		return linkCreator.getLink();
	}


}
