package edu.uiuc.astgen.basic;

import org.eclipse.jdt.core.dom.*;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.edsg.IIterationLink;
import edu.uiuc.edsg.LinkCreator;

public class VariableDeclarationFragmentGenerator extends
		ASTNodeGeneratorBase<VariableDeclarationFragment> {

	private ASTNodeGeneratorBase<SimpleName> _nameGenerator;
	private ASTNodeGeneratorBase<? extends Expression> optionalInitializer;
	
	public VariableDeclarationFragmentGenerator(
			ASTNodeGeneratorBase<SimpleName> nameGenerator) {	
		super();
		_nameGenerator = nameGenerator;
	}
	
	@Override
	protected IIterationLink getInitializedIterator() {
		LinkCreator lc = new LinkCreator(LinkCreator.LinkType.COMBINATORIC);
		lc.add(_nameGenerator);
		if (optionalInitializer != null)
			lc.add(optionalInitializer);
		return lc.getLink();
		//return LinkCreator.createCombinatoricLink(_nameGenerator); // add generators if needed
	}
	
	@Override
	protected VariableDeclarationFragment generateCurrent() {
		VariableDeclarationFragment vdf = super.getAST().newVariableDeclarationFragment();
		vdf.setName(_nameGenerator.current());
		if (optionalInitializer != null)
			vdf.setInitializer(optionalInitializer.current());
		return vdf;
	}

	public ASTNodeGeneratorBase<? extends Expression> getOptionalInitializer() {
		return optionalInitializer;
	}

	public void setOptionalInitializer(
			ASTNodeGeneratorBase<? extends Expression> optionalInitializer) {
		this.optionalInitializer = optionalInitializer;
	}
	
}
