package edu.uiuc.astgen.classrelationships;

import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import edu.uiuc.astgen.ASTNodeGeneratorBase;
import edu.uiuc.astgen.basic.ClassGenerator;
import edu.uiuc.edsg.CompositeGenerator;
import edu.uiuc.edsg.IGenerator;
import edu.uiuc.edsg.IIterationLink;
import edu.uiuc.edsg.LinkCreator;

/** 
 * Generates pairs of type declarations related in various ways  
 */
public class RelatedPairGenerator extends CompositeGenerator<RelatedPair> {	
	private IGenerator<TypeDeclaration> classAGen;
	private IGenerator<TypeDeclaration> classBGen;
	private IGenerator<RelationshipSpecification> builderGen;
	
	public RelatedPairGenerator() {
		
	}

	@Override
	protected IIterationLink getInitializedIterator() {
		if (classAGen == null) {
			classAGen = new ClassGenerator("A");
		}
		if (classBGen == null) {
			classBGen = new ClassGenerator("B");
		}
		if (builderGen == null) {
			builderGen = new RelationshipSpecificationGenerator();
		}
		
		return LinkCreator.createCombinatoricLink(classAGen, classBGen, builderGen);		
	}

	public void setBuilderGen(IGenerator<RelationshipSpecification> builderGen) {
		this.builderGen = builderGen;
	}	
	public void setClassAGen(IGenerator<TypeDeclaration> classAGen) {
		this.classAGen = classAGen;
	}
	public void setClassBGen(IGenerator<TypeDeclaration> classBGen) {
		this.classBGen = classBGen;
	}
	
	@Override
	protected RelatedPair generateCurrent() {
		AST ast = ASTNodeGeneratorBase.ast();
		// copy type declarations to keep from using the same ones for each generation
		TypeDeclaration classA = (TypeDeclaration)ASTNode.copySubtree(ast, classAGen.current());
		TypeDeclaration classB = (TypeDeclaration)ASTNode.copySubtree(ast, classBGen.current());
		RelationshipSpecification relSpec = builderGen.current();
		
		relSpec.getBuilder().build(classA, classB);
		
		return new RelatedPair(classA, classB, relSpec); 
	}
}
